@@ -1,11 +1,13 @@
-from .monads.either import DoEither, Either  # noqa
+from .monads.either import DoEither, DoEitherN, Either  # noqa
+from .monads.either import _ as _e  # noqa
 from .monads.either import do as do_either  # noqa
 
 right = Either.right
 left = Either.left
 
 
-from .monads.maybe import DoMaybe, Maybe  # noqa
+from .monads.maybe import DoMaybe, DoMaybeN, Maybe  # noqa
+from .monads.maybe import _ as _m  # noqa
 from .monads.maybe import do as do_maybe  # noqa
 
 maybe = Maybe.of
@@ -33,4 +35,4 @@ pynction2 = Function2.decorator
 pynction3 = Function3.decorator
 pynction4 = Function4.decorator
 
-from .version import __version__  # noqa
+__version__ = "0.3.2"
